@@ -1,7 +1,5 @@
 /**
  * @file   adin_file.c
- * @author Akinobu LEE
- * @date   Sun Feb 13 13:31:20 2005
  *
  * <JA>
  * @brief  եϡWAV/RAWե뤪ɸϤɤ߹
@@ -55,13 +53,16 @@
  * In this file, assume sizeof(int)=4, sizeof(short)=2
  * </EN>
  *
- * $Revision:$
+ * @author Akinobu LEE
+ * @date   Sun Feb 13 13:31:20 2005
+ *
+ * $Revision: 1.8 $
  * 
  */
 /*
- * Copyright (c) 1991-2005 Kawahara Lab., Kyoto University
+ * Copyright (c) 1991-2007 Kawahara Lab., Kyoto University
  * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
- * Copyright (c) 2005      Julius project team, Nagoya Institute of Technology
+ * Copyright (c) 2005-2007 Julius project team, Nagoya Institute of Technology
  * All rights reserved
  */
 
@@ -84,6 +85,9 @@ static boolean has_pre;		///< TRUE if pre_data is available
 
 static unsigned int sfreq;	///< Sampling frequency in Hz, specified by adin_standby()
 
+static char speechfilename[MAXPATHLEN];	///< Buffer to hold input file name
+static char *stdin_buf = NULL;
+
 /* read .wav data with endian conversion */
 /* (all .wav datas are in little endian) */
 
@@ -102,7 +106,7 @@ static boolean
 myread(void *buf, size_t unitbyte, int unitnum, FILE *fp)
 {
   int tmp;
-  if ((tmp = myfread(buf, unitbyte, unitnum, fp)) < unitnum) {
+  if ((tmp = fread(buf, unitbyte, unitnum, fp)) < unitnum) {
     return(FALSE);
   }
 #ifdef WORDS_BIGENDIAN
@@ -111,7 +115,7 @@ myread(void *buf, size_t unitbyte, int unitnum, FILE *fp)
   return(TRUE);
 }
 /// Abbreviation for header reading
-#define MYREAD(A,B,C,D)  if (!myread(A, B, C, D)) {j_printerr("adin_file: file is corrupted\n"); return -1;}
+#define MYREAD(A,B,C,D)  if (!myread(A, B, C, D)) {jlog("Error: adin_file: file is corrupted\n"); return -1;}
 
 /** 
  * @brief  Parse header part of a WAVE file to prepare for data reading
@@ -143,8 +147,8 @@ setup_wav(FILE *fp)
       dummy[1] != 'A' ||
       dummy[2] != 'V' ||
       dummy[3] != 'E') {
-    j_printerr("adin_file: WAVE header not found, file corrupted?\n");
-    fclose_readfile(fp); return FALSE;
+    jlog("Error: adin_file: WAVE header not found, file corrupted?\n");
+    return FALSE;
   }
   /* format chunk: "fmt " */
   MYREAD(dummy, 1, 4, fp);
@@ -152,8 +156,8 @@ setup_wav(FILE *fp)
       dummy[1] != 'm' ||
       dummy[2] != 't' ||
       dummy[3] != ' ') {
-    j_printerr("adin_file: fmt chunk not found, file corrupted?\n");
-    fclose_readfile(fp); return FALSE;
+    jlog("Error: adin_file: fmt chunk not found, file corrupted?\n");
+    return FALSE;
   }
   /* 4byte: byte size of this part */
   MYREAD(&len, 4, 1, fp);
@@ -161,38 +165,38 @@ setup_wav(FILE *fp)
   /* 2byte: data format */
   MYREAD(&s, 2, 1, fp);
   if (s != 1) {
-    j_printerr("adin_file: data format != PCM (id=%d)\n", s);
-    fclose_readfile(fp); return FALSE;
+    jlog("Error: adin_file: data format != PCM (id=%d)\n", s);
+    return FALSE;
   }
   /* 2byte: channel num */
   MYREAD(&s, 2, 1, fp);
   if (s >= 2) {
-    j_printerr("adin_file: channel num != 1 (%d)\n", s);
-    fclose_readfile(fp); return FALSE;
+    jlog("Error: adin_file: channel num != 1 (%d)\n", s);
+    return FALSE;
   }
   /* 4byte: sampling rate */
   MYREAD(&i, 4, 1, fp);
   if (i != sfreq) {
-    j_printerr("adin_file: sampling rate != %d (%d)\n", sfreq, i);
-    fclose_readfile(fp); return FALSE;
+    jlog("Error: adin_file: sampling rate != %d (%d)\n", sfreq, i);
+    return FALSE;
   }
   /* 4byte: bytes per second */
   MYREAD(&i, 4, 1, fp);
   if (i != sfreq * sizeof(SP16)) {
-    j_printerr("adin_file: bytes per second != %d (%d)\n", sfreq * sizeof(SP16), i);
-    fclose_readfile(fp); return FALSE;
+    jlog("Error: adin_file: bytes per second != %d (%d)\n", sfreq * sizeof(SP16), i);
+    return FALSE;
   }
   /* 2bytes: bytes per frame ( = (bytes per sample) x channel ) */
   MYREAD(&s, 2, 1, fp);
   if (s != 2) {
-    j_printerr("adin_file: (bytes per sample) x channel != 2 (%d)\n", s);
-    fclose_readfile(fp); return FALSE;
+    jlog("Error: adin_file: (bytes per sample) x channel != 2 (%d)\n", s);
+    return FALSE;
   }
   /* 2bytes: bits per sample */
   MYREAD(&s, 2, 1, fp);
   if (s != 16) {
-    j_printerr("adin_file: bits per sample != 16 (%d)\n", s);
-    fclose_readfile(fp); return FALSE;
+    jlog("Error: adin_file: bits per sample != 16 (%d)\n", s);
+    return FALSE;
   }
   /* skip rest */
   if (len > 16) {
@@ -242,17 +246,27 @@ adin_file_open(char *filename)	/* NULL for standard input */
   char dummy[4];
 
   if (filename != NULL) {
-    if ((fp = fopen_readfile(filename)) == NULL) {
-      j_printerr("failed to open %s\n",filename);
+    if ((fp = fopen(filename, "rb")) == NULL) {
+      jlog("Error: adin_file: failed to open %s\n",filename);
       return(FALSE);
     }
   } else {
     fp = stdin;
+#if defined(_WIN32) && !defined(__CYGWIN32__) && !defined(__MINGW32__)
+    if (_setmode( _fileno( stdin ), _O_BINARY ) == -1) {
+      jlog("Error: adin_file: _setmode() failed\n");
+    }
+#endif
+    if (stdin_buf == NULL) {
+      stdin_buf = (char *)mymalloc(BUFSIZ);
+      setvbuf(stdin, stdin_buf, _IOFBF, BUFSIZ);
+    }
   }
+    
   /* check first 4 byte to detect Microsoft WAVE format */
-  if (myfread(dummy, 1, 4, fp) < 4) {
-    j_printerr("Error: size less than 4 bytes?\n",filename);
-    fclose_readfile(fp);
+  if (fread(dummy, 1, 4, fp) < 4) {
+    jlog("Error: adin_file: size less than 4 bytes?\n",filename);
+    fclose(fp);
     return(FALSE);
   }
   if (dummy[0] == 'R' &&
@@ -263,8 +277,8 @@ adin_file_open(char *filename)	/* NULL for standard input */
     wav_p = TRUE;
     has_pre = FALSE;
     if (setup_wav(fp) == FALSE) {
-      j_printerr("Error: failed to read %s as a wav file\n",filename);
-      fclose_readfile(fp);
+      jlog("Error: adin_file: error in parsing wav header at %s\n",filename);
+      fclose(fp);
       return(FALSE);
     }
   } else {
@@ -290,56 +304,13 @@ adin_file_close()
   FILE *fp;
 
   fp = gfp;
-  if (fclose_readfile(fp) != 0) {
-    perror("adin_file_close: fclose_readfile");
+  if (fclose(fp) != 0) {
+    jlog("Error: adin_file: failed to close file\n");
     return FALSE;
   }
  return TRUE; 
 }
 
-
-
-/* get 1 line input from stdin with prompt */
-/* return value: newly allocated buffer */
-/* repeat if no input, and */
-/* returns NULL on EOF */
-/** 
- * Get one file name from stdin with a prompt .  Blank line is omitted.
- * 
- * @param prompt [in] prompt string
- * 
- * @return newly allocated buffer filled with the input.
- */
-char *
-get_line(char *prompt)
-{
-  char *buf = NULL;
-  char *p;
-
-  buf = (char *)mymalloc(500);
-  do {
-    j_printerr("%s",prompt);
-    if (fgets(buf, 500, stdin) == NULL) {
-      free(buf);
-      return(NULL);
-    }
-  } while (!buf[0]);		/* loop till some input */
-  /* chop last newline */
-  p = buf + strlen(buf) - 1;
-  if (p >= buf && *p == '\n') {
-    *(p --) = '\0';
-  }
-  if (p >= buf && *p == '\r') {
-    *(p --) = '\0';
-  }
-  /* chop last space */
-  while(p >= buf && *p == ' ') {
-    *(p --) = '\0';
-  }
-
-  return(buf);
-}
-
 static boolean from_file;	///< TRUE if list file is used to read input filename
 static FILE *fp_list;		///< File pointer used for the listfile
 
@@ -358,7 +329,7 @@ adin_file_standby(int freq, void *arg)
   if (fname != NULL) {
     /* read input filename from file */
     if ((fp_list = fopen(fname, "r")) == NULL) {
-      j_printerr("failed to open %s\n", fname);
+      jlog("Error: adin_file: failed to open %s\n", fname);
       return(FALSE);
     }
     from_file = TRUE;
@@ -379,40 +350,50 @@ adin_file_standby(int freq, void *arg)
  * will be read from the listfile.  Otherwise, the
  * filename will be obtained from stdin.  Then the file will be opened here.
  * 
+ * @param filename [in] file name to open or NULL for prompt
+ * 
  * @return TRUE on success, FALSE on failure.
  */
 boolean
-adin_file_begin()
+adin_file_begin(char *filename)
 {
-  char *speechfilename;
   boolean readp;
 
+  if (filename != NULL) {
+    /* open the file and exit with its status */
+    if (adin_file_open(filename) == FALSE) {
+      jlog("Error: adin_file: failed to read speech data: \"%s\"\n", filename);
+      return FALSE;
+    }
+    jlog("Stat: adin_file: input speechfile: %s\n", filename);
+    strcpy(speechfilename, filename);
+    return TRUE;
+  }
+
   /* ready to read next input */
   readp = FALSE;
   while(readp == FALSE) {
     if (from_file) {
       /* read file name from listfile */
-      speechfilename = (char *)mymalloc(500);
       do {
-	if (getl_fp(speechfilename, 500, fp_list) == NULL) { /* end of input */
-	  free(speechfilename);
+	if (getl_fp(speechfilename, MAXPATHLEN, fp_list) == NULL) { /* end of input */
 	  fclose(fp_list);
 	  return(FALSE); /* end of input */
 	}
       } while (speechfilename[0] == '#'); /* skip comment */
     } else {
       /* read file name from stdin */
-      speechfilename = get_line("enter filename->");
-      if (speechfilename == NULL) return (FALSE);	/* end of input */
+      if (get_line_from_stdin(speechfilename, MAXPATHLEN, "enter filename->") == NULL) {
+	return (FALSE);	/* end of input */
+      }
     }
     /* open input file */
     if (adin_file_open(speechfilename) == FALSE) {
-      j_printerr("Error in reading speech data: \"%s\"\n",speechfilename);
+      jlog("Error: adin_file: failed to read speech data: \"%s\"\n", speechfilename);
     } else {
-      j_printf("\ninput speechfile: %s\n",speechfilename);
+      jlog("Stat: adin_file: input speechfile: %s\n", speechfilename);
       readp = TRUE;
     }
-    free(speechfilename);
   }
   return TRUE;
 }
@@ -434,7 +415,15 @@ adin_file_read(SP16 *buf, int sampnum)
   fp = gfp;
   
   if (wav_p) {
-    cnt = myfread(buf, sizeof(SP16), sampnum, fp);
+    cnt = fread(buf, sizeof(SP16), sampnum, fp);
+    if (cnt == 0) {
+      if (feof(fp)) return -1; /* EOF */
+      if (ferror(fp)) {
+	jlog("Error: adin_file: an error occured while reading file\n");
+	adin_file_close();
+	return -2; /* error */
+      }
+    }
     if (nowlen + cnt > maxlen) {
       cnt = maxlen - nowlen;
     }
@@ -443,20 +432,28 @@ adin_file_read(SP16 *buf, int sampnum)
     if (has_pre) {
       buf[0] = pre_data[0]; buf[1] = pre_data[1];
       has_pre = FALSE;
-      cnt = myfread(&(buf[2]), sizeof(SP16), sampnum - 2, fp);
-      if (cnt > 0) cnt += 2;
+      cnt = fread(&(buf[2]), sizeof(SP16), sampnum - 2, fp);
+      if (cnt == 0) {
+	if (feof(fp)) return -1; /* EOF */
+	if (ferror(fp)) {
+	  jlog("Error: adin_file: an error occured file reading file\n");
+	  adin_file_close();
+	  return -2; /* error */
+	}
+      }
+      cnt += 2;
     } else {
-      cnt = myfread(buf, sizeof(SP16), sampnum, fp);
+      cnt = fread(buf, sizeof(SP16), sampnum, fp);
+      if (cnt == 0) {
+	if (feof(fp)) return -1; /* EOF */
+	if (ferror(fp)) {
+	  jlog("Error: adin_file: an error occured file reading file\n");
+	  adin_file_close();
+	  return -2; /* error */
+	}
+      }
     }
   }
-  if (cnt == 0) {		/* error or EOF */
-    if (myfeof(fp) == 1) {		/* EOF */
-      return -1;
-    }
-    perror("adin_file_read");
-    adin_file_close();
-    return -2;		/* error */
-  }
   /* all .wav data are in little endian */
   /* assume .raw data are in big endian */
 #ifdef WORDS_BIGENDIAN
@@ -499,19 +496,23 @@ adin_stdin_standby(int freq, void *arg)
 /** 
  * @brief  Begin reading audio data from stdin
  *
+ * @param pathname [in] dummy
+ *
  * @return TRUE on success, FALSE on failure.
  */
 boolean
-adin_stdin_begin()
+adin_stdin_begin(char *pathname)
 {
   if (feof(stdin)) {		/* already reached the end of input stream */
+    jlog("Error: adin_stdin: stdin reached EOF\n");
     return FALSE;		/* terminate search here */
   } else {
     /* open input stream */
     if (adin_file_open(NULL) == FALSE) {
-      j_printerr("Error in reading speech data from stdin\n");
+      jlog("Error: adin_stdin: failed to read speech data from stdin\n");
+      return FALSE;
     }
-    j_printf("Reading wavedata from stdin...\n");
+    jlog("Stat: adin_stdin: reading wavedata from stdin...\n");
   }
   return TRUE;
 }
@@ -530,24 +531,39 @@ adin_stdin_read(SP16 *buf, int sampnum)
   int cnt;
 
   if (wav_p) {
-    cnt = myfread(buf, sizeof(SP16), sampnum, stdin);
+    cnt = fread(buf, sizeof(SP16), sampnum, stdin);
+    if (cnt == 0) {
+      if (feof(stdin)) return -1; /* EOF */
+      if (ferror(stdin)) {
+	jlog("Error: adin_stdin: an error occured while reading stdin\n");
+	return -2; /* error */
+      }
+    }
   } else {
     if (has_pre) {
       buf[0] = pre_data[0]; buf[1] = pre_data[1];
       has_pre = FALSE;
       cnt = fread(&(buf[2]), sizeof(SP16), sampnum - 2, stdin);
-      if (cnt > 0) cnt += 2;
+      if (cnt == 0) {
+	if (feof(stdin)) return -1; /* EOF */
+	if (ferror(stdin)) {
+	  jlog("Error: adin_stdin: an error occured while reading stdin\n");
+	  return -2; /* error */
+	}
+      }
+      cnt += 2;
     } else {
       cnt = fread(buf, sizeof(SP16), sampnum, stdin);
+      if (cnt == 0) {
+	if (feof(stdin)) return -1; /* EOF */
+	if (ferror(stdin)) {
+	  jlog("Error: adin_stdin: an error occured while reading stdin\n");
+	  return -2; /* error */
+	}
+      }
     }
   }
-  if (cnt == 0) {     
-    if (ferror(stdin)) {		/* error */
-      perror("adin_stdin_read");
-      return -2;		/* error */
-    }
-    return -1;			/* EOF */
-  }
+
   /* all .wav data are in little endian */
   /* assume .raw data are in big endian */
 #ifdef WORDS_BIGENDIAN
@@ -557,3 +573,28 @@ adin_stdin_read(SP16 *buf, int sampnum)
 #endif
   return cnt;
 }
+
+/** 
+ * 
+ * A tiny function to get current input raw speech file name.
+ * 
+ * @return string of current input speech file.
+ * 
+ */
+char *
+adin_file_get_current_filename()
+{
+  return(speechfilename);
+}
+/** 
+ * 
+ * A tiny function to get current input raw speech file name.
+ * 
+ * @return string of current input speech file.
+ * 
+ */
+char *
+adin_stdin_input_name()
+{
+  return("stdin");
+}
