@@ -1,7 +1,5 @@
 /**
  * @file   rdhmmlist.c
- * @author Akinobu LEE
- * @date   Wed Feb 16 04:04:23 2005
  * 
  * <JA>
  * @brief  HMMListեɤ߹
@@ -40,13 +38,16 @@
  * produces error.
  * </EN>
  * 
- * $Revision:$
+ * @author Akinobu LEE
+ * @date   Wed Feb 16 04:04:23 2005
+ *
+ * $Revision: 1.3 $
  * 
  */
 /*
- * Copyright (c) 1991-2005 Kawahara Lab., Kyoto University
+ * Copyright (c) 1991-2007 Kawahara Lab., Kyoto University
  * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
- * Copyright (c) 2005      Julius project team, Nagoya Institute of Technology
+ * Copyright (c) 2005-2007 Julius project team, Nagoya Institute of Technology
  * All rights reserved
  */
 
@@ -79,16 +80,21 @@ rdhmmlist(FILE *fp, HTK_HMM_INFO *hmminfo)
   n = 0;
   while (getl(buf, MAXLINEINHMMLIST, fp) != NULL) {
     n++;
-    lname = first_token(buf);
-    if (strlen(lname) >= MAX_HMMNAME_LEN) {
-      j_error("Error: HMMList: %d: name too long: \"%s\"\n", n, lname);
+    if ((lname = strtok(buf, DELM)) == NULL) {
+      jlog("Error: rdhmmlist: failed to parse, corrupted or invalid data?\n");
+      return FALSE;
     }
-    pname = next_token_if_any();
+    if (strlen(lname) >= MAX_HMMNAME_LEN) {
+      jlog("Error: rdhmmlist: %d: name too long: \"%s\"\n", n, lname);
+      jlog("Error: rdhmmlist: try increase value of MAX_HMMNAME_LEN\n");
+      return FALSE;
+    }
+    pname = strtok(NULL, DELM);
     if (pname == NULL) {
       /* 1 column */
       mapped = htk_hmmdata_lookup_physical(hmminfo, lname);
       if (mapped == NULL) {
-	j_printerr("Error: HMMList: line %d: physical HMM \"%s\" not found\n", n, lname);
+	jlog("Error: rdhmmlist: line %d: physical HMM \"%s\" not found\n", n, lname);
 	ok_flag = FALSE;
 	continue;
       }
@@ -96,31 +102,33 @@ rdhmmlist(FILE *fp, HTK_HMM_INFO *hmminfo)
       /* 2 column */
       mapped = htk_hmmdata_lookup_physical(hmminfo, pname);
       if (strlen(pname) >= MAX_HMMNAME_LEN) {
-	j_error("Error: HMMList: %d: name too long: \"%s\"\n", n, pname);
+	jlog("Error: rdhmmlist: %d: name too long: \"%s\"\n", n, pname);
+	jlog("Error: rdhmmlist: please increase MAX_HMMNAME_LEN (%d) and re-compile\n", MAX_HMMNAME_LEN);
+	return FALSE;
       }
       if (mapped == NULL) {
-	j_printerr("Error: HMMList: line %d: physical HMM \"%s\" not found\n", n, pname);
+	jlog("Error: rdhmmlist: line %d: physical HMM \"%s\" not found\n", n, pname);
 	ok_flag = FALSE;
 	continue;
       }
     }
     /* create new HMM_Logical */
-    new = (HMM_Logical *)mybmalloc(sizeof(HMM_Logical));
-    new->name = mybstrdup(lname);
+    new = (HMM_Logical *)mybmalloc2(sizeof(HMM_Logical), &(hmminfo->lroot));
+    new->name = mybstrdup2(lname, &(hmminfo->lroot));
     new->is_pseudo = FALSE;
     new->body.defined = mapped;
     new->next = hmminfo->lgstart;
     hmminfo->lgstart = new;
     /* add index to search index tree */
     if (hmminfo->logical_root == NULL) {
-      hmminfo->logical_root = aptree_make_root_node(new);
+      hmminfo->logical_root = aptree_make_root_node(new, &(hmminfo->lroot));
     } else {
       match = aptree_search_data(new->name, hmminfo->logical_root);
-      if (strmatch(match->name, new->name)) {
-	j_printerr("Error: HMMList: line %d: logical HMM \"%s\" duplicated\n", n, new->name);
+      if (match != NULL && strmatch(match->name, new->name)) {
+	jlog("Error: rdhmmlist: line %d: logical HMM \"%s\" duplicated\n", n, new->name);
 	ok_flag = FALSE;
       } else {
-	aptree_add_entry(new->name, new, match->name, &(hmminfo->logical_root));
+	aptree_add_entry(new->name, new, match->name, &(hmminfo->logical_root), &(hmminfo->lroot));
       }
     }
     
@@ -128,5 +136,6 @@ rdhmmlist(FILE *fp, HTK_HMM_INFO *hmminfo)
 
   hmminfo->totallogicalnum = n;
   free(buf);
+
   return(ok_flag);
 }
